<CsoundSynthesizer>

<CsOptions>
-m0 -d -L stdin -odac -M hammond_in
</CsOptions>

<CsInstruments>
/***************************************************************************************
 Tone Wheel Organ with Rotating Speaker
  by Pete Goodeve, based on an original by Hans Mikelson
 Two Manuals, Vibrato, Percussion, Spring Reverb, and Leslie with Overdrive

 v8.0 -- 2013/2/4  Major revisions: Csound 5.18, added MIDI control, new algorithms etc.
 v8.1 -- 2018/11/17 Revised Percussion & CC numbers, more MIdi controls, etc..
******************************************************************************************/

sr     = 24000	; should be adequate for tonewheels
ksmps  = 120

0dbfs = 1.0

nchnls = 2

; Midi Channel Assignments -- you can switch the channels used here 
giChanSwell = 1		; Upper Manual MIDI channel
giChanGreat = 2		; Lower Manual MIDI channel
giChanLeslie = 3	; leslie speed control by MIDI (also Vibrato controllers)

; Disconnect all MIDI first
ichn = 1
assignlp:
	massign	 ichn, 0
	loop_le	 ichn, 1, 16, assignlp
	pgmassign 0, 0

; Assign channels to instruments
massign giChanLeslie, "Rotor"
massign giChanSwell, "ManualS"
massign giChanGreat, "ManualG"


;---------- Score Function Table Defaults: ---------
; (i.e. they can be supplied by the score, with function-table numbers as below,
; but these defaults will apply otherwise)


; Swell Pedal Characteristic [table 2]:
; -- possibly good for an actual pedal:
;giSwell ftgen 2, 0, 129, 7, \
;          0.001, 3, 0.01, 30, 0.5, 37, 0.8, 58, 1, 1, 1
;; Plain exponential version better for slider (half power ~110):
giSwell ftgen 2, 0, 129, 5, 0.01, 128, 1.0

; Leslie Speeds (Revs/sec) for control via MIDI [Tables 5,6]:
; ... Values gleaned and extrapolated from statements on the web...
;  Adjust to taste
; Horn should spin slightly faster than the Bass

giLeslieHorn ftgen 5, 0, 8, -17, \
	0, 0, \
	1, 0.8, \
	2, 0.85, \
	3, 1.4, \
	4, 6.7, \
	5, 6.8, \
	6, 7.5, \
	7, 9
	
giLeslieBass ftgen 6, 0, 8, -17, \
	0, 0, \
	1, 0.7, \
	2, 0.8, \
	3, 1.2, \
	4, 5.7, \
	5, 6.6, \
	6, 7.0, \
	7, 8
	
;---------- Misc. Adjustable Parameters: ---------

giKeyVol = 0.125	; This should be set so that chords don't have peaks much above 1.0
					; because the distortion stage expects peaks to be that.
giOutGain = 1		; This can be adjusted to suit the output peaks to the system:
					; if there are faders later in series, it probably should be > 1.0
					; (This will of course give "Out of Range" reports if these are
					; enabled, but it is only the final conversion to DAC that can
					; actually overrange.)

;Percussion Parameters -- change to taste
; -- adjustable via instr PercParam or MIDI ch 3 CC22..24
;Amplitude scaling
giPercAmp init 2.0
;Percussion decay ("RC") time constants
;; These values sound plausible:
giPercFast init 0.2
giPercSlow init 3.0

; Keyclick  -- adjust to taste (default value seems good)
; (The main cause of the click is picking up the contimuous sinewaves
;  at a random point -- mimimized by a slower rise-time.  A white-noise
;  click sound can also be added if desired.  In this version, the
;  white noise is modulated by the tone output, so is proportional 
;  to drawbar settings.)
giClickRise = 0.02	; set to zero for maximum contact click -- >=0.02 to minimize
giClickNoise = 0	; "Hash" amplitude:  <1.0 if you want it...
giClickBeta = 0.6	; low-pass for hash -- the closer to 1 the lower...

; Swell Pedal Initial position (50% power for exponential map)
ctrlinit giChanSwell, 11, 110
ctrlinit giChanGreat, 11, 110

; Drawbar MIDI block starts from Controller number:
giDBControlBase = 20	;ch. 1, ch.2
giPercSwBase = 81	; OFF/ON, 2nd/3rd, Slow/Fast -- ch. 1
giPercParam = 22		; 22..24 => scale, fast, slow -- ch. 3 (Leslie)
giVibrSwCC = 80	; OFF/ON, -- ch. 1
giVibratoCC = 20	; MIDI control of Vibrato/Chorus -- ch. 3 (Leslie)
giOverdriveCC = 21	; MIDI control of Distortion -- ch. 3 (Leslie)
giReverbBase = 25		; 25,26 => Spring, Room Reverb -- ch. 3 (Leslie)

; Leslie Characteristics:
giLeslieBase = 21	; note number (chan 3) that will 'stop' the Leslie rotors
                 	; Leslie speed changed by the following 7 keys (via table)
giRampTimeHigh = 1	; Time (secs) for Leslie top rotor to come to speed
giRampTimeLow = 2	; Time (secs) for Leslie lower rotor to come to speed

;Leslie Freq Range scaling:
; (can be changed via instr "Control" p3=12)
;; just flat for now...:
giHiDirect = 1.0
giHiRefl = 1.0
giMidDirect = 1.0
giMidRefl = 1.0
;giLowAll = 1.0
giLowAll = 0.5


;---------- Fixed Parameters: ---------

; Fixed Organ manual keyboard range (0 == 2 octaves below middle-C)
gikeylo = 0		; == MIDI #36
gikeyhi = 60	; == MIDI #96

gkOne init 1	;; literal constants seem(ed?) to have a bug in many ops...
giMinDelay init 1.0/kr	;; so does v.short delay

; These instruments must not have more than one instance:
maxalloc "Vibrato", 1
maxalloc "Spring_Reverb", 1
maxalloc "Overdrive", 1	
maxalloc "Leslie", 1
maxalloc "Output", 1	; Audio Out/Chain Terminator

; Vibrato Scanner Delay Line Length:
giVibrLine = 1.1	;millisec

; ------- (Fixed) Function Tables: ---------
; (Table numbers 3, 4, 9, 11, 12, 13, 14, 20, 21, 22
;  -- only modify these if you're REALLY feeling experimental...)

; Tone Wheel Frequencies [Table 20]:
; (from wheel teeth and gearing on actual Hammond -- 1200 RPM main shaft)
giTones ftgen 20, 0, 128, -17, \
	0, 32.69230768, \
	1, 34.63414636, \
	2, 36.71232876, \
	3, 38.88888888, \
	4, 41.2, \
	5, 43.63636364, \
	6, 46.25, \
	7, 49, \
	8, 51.89189188, \
	9, 55, \
	10, 58.26086956, \
	11, 61.71428572, \
	12, 65.38461536, \
	13, 69.26829272, \
	14, 73.42465752, \
	15, 77.77777776, \
	16, 82.4, \
	17, 87.27272728, \
	18, 92.5, \
	19, 98, \
	20, 103.78378376, \
	21, 110, \
	22, 116.52173912, \
	23, 123.42857144, \
	24, 130.76923072, \
	25, 138.53658544, \
	26, 146.84931504, \
	27, 155.55555552, \
	28, 164.8, \
	29, 174.54545456, \
	30, 185, \
	31, 196, \
	32, 207.56756752, \
	33, 220, \
	34, 233.04347824, \
	35, 246.85714288, \
	36, 261.53846144, \
	37, 277.07317088, \
	38, 293.69863008, \
	39, 311.11111104, \
	40, 329.6, \
	41, 349.09090912, \
	42, 370, \
	43, 392, \
	44, 415.13513504, \
	45, 440, \
	46, 466.08695648, \
	47, 493.71428576, \
	48, 523.07692288, \
	49, 554.14634176, \
	50, 587.39726016, \
	51, 622.22222208, \
	52, 659.2, \
	53, 698.18181824, \
	54, 740, \
	55, 784, \
	56, 830.27027008, \
	57, 880, \
	58, 932.17391296, \
	59, 987.42857152, \
	60, 1046.15384576, \
	61, 1108.29268352, \
	62, 1174.79452032, \
	63, 1244.44444416, \
	64, 1318.4, \
	65, 1396.36363648, \
	66, 1480, \
	67, 1568, \
	68, 1660.54054016, \
	69, 1760, \
	70, 1864.34782592, \
	71, 1974.85714304, \
	72, 2092.30769152, \
	73, 2216.58536704, \
	74, 2349.58904064, \
	75, 2488.88888832, \
	76, 2636.8, \
	77, 2792.72727296, \
	78, 2960, \
	79, 3136, \
	80, 3321.08108032, \
	81, 3520, \
	82, 3728.69565184, \
	83, 3949.71428608, \
	84, 4189.09090944, \
	85, 4440, \
	86, 4704, \
	87, 4981.62162048, \
	88, 5280, \
	89, 5593.04347776, \
	90, 5924.57142912

; Amplitude scaling of frequencies -- currently disabled (just flat)
giScaling ftgen 21, 0, 128, 7, 1.0, 60, 1.0, 30, 1.0, 38, 1.0   ;no fall-off at all!
;giScaling ftgen 21, 0, 128, 5, 5.0, 90, 1.0, 38, 0.3   ;faster fall-off when folded...
;; Match to real Hammond C-sample:
;giScaling ftgen 21, 0, 128, 5, 10.0, 60, 2.0, 30, 0.3, 38, 0.02   ;faster fall-off when folded...

; Relation of DrawBar position to harmonic amplitude [Table 22]
giDBresp = 3	; 0 is straight line, ~5 is x2/step
; Csound bug -- table entries have to be *longer* than 9...!)
;giDrawbar ftgen 22, 0, 16, -16, 0.0, 8, giDBresp, 8.0	; (8 segs 1..9)
;; Use explicit 3dB steps instead of weird GEN16... (also normalized):
; OK... realizing it's 3dB/step of *power*, not voltage...!
giDrawbar ftgen 22, 0, 16, 7, \
     0.0, 1, 1.4142, 1, 2.0, 1, 2.8284, 1, 4.0, 1, 5.6569, 1, 8.0, 1, 11.3137, 1, 16.0, \
     1,16.0  ;; yes -- this last entry is needed, even though it returns as zero!
; According to someone's actual measurements, these are the actual dB (power) levels:
;  [OFF], -32.1, -26.1, -20.0, -15.2, -10.5, -6.0, -3.0, 0.0  -- giving:
;giDrawbar ftgen 22, 0, 16, 7, \
;     0.0, 1, 0.0248, 1, 0.0495, 1, 0.1000, 1, 0.1738, 1, 0.2985, 1, 0.5012, 1, 0.7079, 1, 1.0000, \
;     1,1.000 

; Tone Wheel waveforms [Tables 11,12]:
giSine1  ftgen 11, 0, 16384, 10, 1, .02, .01	; -- also Low freq Leslie
giSine2  ftgen 12, 0, 16384, 10, 1, 0, .2, 0, .1, 0, .05, 0, .02


;Triangular Scanner Sweep [Table 9]:
;; Scan is not actually quite triangular!:
giScanner ftgen 9, 0, 16, 7,  \
  0, 1,1,  1,3,  1,6,  1, 9,  1,12,  1,15,  1,17,  1,18,\
  1,17,  1,15,  1,12,  1,9,  1,6,  1,3,  1,1,  1,0 

; Leslie Horn Characteristics [Tables 3,4,13,14]:
giLhidef  ftgen 3, 0, 1024, 8, .95, 24, .85, 24, 1, 24, .85, 24, 1, 24, .85, 248, .9, 72, .8, 72, 1, 72, .8, 72, .9, 248, .85, 24, 1, 24, .85, 24, 1, 24, .85, 24, .95
giLmiddef  ftgen 4, 0, 1024, 8, .95, 48, .85, 96, .75, 240, .8, 64, 1, 128, 1, 64, .8, 240, .75, 96, .85, 48, .95
giLhinodef  ftgen 13, 0, 1024, 8, .2, 440, .4, 72, 1, 72, .4, 440, .2
giLmidnodef  ftgen 14, 0, 1024, 8, .4, 320, .6, 64, 1, 256, 1, 64, .6, 320, .4
giLlowrot  ftgen 15, 0, 1024, 8, .6, 320, .8, 64, 1, 256, 1, 64, .8, 320, .6

giRvbRamp ftgen 0, 0, 129, 7, 0, 20, 0.5, 108, 128/127	; Room reverb is negligible below 0.6 (needs guard point!)
;...table hacked to (mostly) counter ctrl7 bug

;--------------------------------


; ------- Global Initialization: ---------

; Leslie stationary by default:
gispeedhorn   init 0
gispeedbass   init 0

givibrS init 0
givibrG init 0

gkSwellPedal init 0.5	; for both manuals
giPercuss2 init 0
giPercuss3 init 0
giPercDecay init 0
gkpercenv init 0	; no percussion unless triggered

gkSpringRvrb	init 0	; range 0..1
gkRoomRvrb		init 0	; 0..0.99  set by Instr 'Conttrol'

gaorgan  init  0                       ;Global send to speaker
gabypass init 0							; Bypass Vibrato
galeft init 0							;Leslie Outputs
garight init 0

giInitVib init 32	;  to match initial instr setting

ctrlinit giChanSwell, \
	giPercSwBase, 0, \
	giPercSwBase+1, 0, \
	giPercSwBase+2, 0 

ctrlinit giChanLeslie, \
	giPercParam,   0, \
	giPercParam+1, 0, \
	giPercParam+2, 0	; lowest end values until set from K/B

ctrlinit giChanLeslie, giVibratoCC, giInitVib
ctrlinit giChanLeslie, giReverbBase, 0, giReverbBase+1, 0


; -------- Audio Processing Instruments: ---------
;        "C2":      FX   Scan  FiltTop Scale Q Buzz 
alwayson "Vibrato", 0.5, 0.67, 10000,   1,   1, 0.05

; Spring Reverb On, but not active by default (level 0..1)
alwayson "Spring_Reverb"

;Overdrive Distortion (param is level = 0..4):
alwayson "Overdrive", 0

;                 Defl  Refl Delay
alwayson "Leslie", 1,   .6,  0.8

;          No reverb by default (use 'Control 10' to set):
alwayson "Output"


/****************************************************************************
 Drawbars, Vibrato On/Off, and Percussion Setting for Upper ("Swell") Manual
 Set by Score Events --  can also be changed by MIDI (via Output instrument)
****************************************************************************/

instr Percussion	;Percussion switching invoke directly or via DrawbarsS
 ; Switches (0/1): p4=>ON/OFF, p5=>2nd/3rd, p6=>Slow/Fast
 pset 0, 0, 0,   0, 0, 0
; prints "Percussion %f %f\n", p4, p5, p6

	giPercDecay init p6
	giPercuss2 init 0
	giPercuss3 init 0
	if p4 > 0 then
		if p5 > 0 then
			giPercuss3 init 1
		else
			giPercuss2 init 1
		endif
	endif

prints "\nPercussion has ampl.: %f fast: %f slow: %f\n", giPercAmp, giPercFast, giPercSlow
prints "\nset Percussion 2nd: %d 3rd: %d slow/fast: %d\n", giPercuss2, giPercuss3, giPercDecay

endin


instr SetPercParams	; used to set i-time vars fron k-time
	prints "Setting Percussion Scale=%f Fast=%f Slow=%f\n", p4, p5, p6
	giPercAmp = p4
	giPercFast = p5
	giPercSlow = p6
endin



instr DrawbarsS
  	
  	; set indicators for missing parameters:
  	pset 0, 0, 0,   0, 0, 0, 0, 0, 0, 0, 0, 0, -999,  -999, 0, 0
  	
prints "Swell Drawbars: %d %d %d %d %d %d %d %d %d   Vibr: %d Perc: %d %d %d\n", \
	p4, p5, p6, p7, p8, p9, p10, p11, p12, p13, p14, p15, p16

gksubfund  init p4
gksub3rd   init p5
gkfund     init p6
gk2nd      init p7
gk3rd      init p8
gk4th      init p9
gk5th      init p10
gk6th      init p11
gk8th      init p12
givibrS    init p13 >= 0 ? p13 : givibrS

if p14 != -999 then
	event_i "i", "Percussion", 0, 0, p14, p15, p16
endif


ctrlinit giChanSwell, \
	giDBControlBase, i(gksubfund), \
	giDBControlBase+1, i(gksub3rd), \
	giDBControlBase+2, i(gkfund), \
	giDBControlBase+3, i(gk2nd), \
	giDBControlBase+4, i(gk3rd), \
	giDBControlBase+5, i(gk4th), \
	giDBControlBase+6, i(gk5th), \
	giDBControlBase+7, i(gk6th), \
	giDBControlBase+8, i(gk8th), \
	giVibrSwCC,        givibrS*127
endin
 
       
/****************************************************************************
 Drawbars and Vibrato On/Off Setting for Lower ("Great") Manual
 Score Events only --  can also be changed by MIDI in ManualS instrument
****************************************************************************/

instr DrawbarsG

  	; set indicators for missing parameters:
  	pset 0, 0, 0,   0, 0, 0, 0, 0, 0, 0, 0, 0, -999

prints "Great Drawbars: %d %d %d %d %d %d %d %d %d   Vibr: %d\n", \
	p4, p5, p6, p7, p8, p9, p10, p11, p12, p13

gksubfundG  init p4
gksub3rdG   init p5
gkfundG     init p6
gk2ndG      init p7
gk3rdG      init p8
gk4thG      init p9
gk5thG      init p10
gk6thG      init p11
gk8thG      init p12
givibrG    init p13 >= 0 ? p13 : givibrG

ctrlinit giChanGreat, \
	giDBControlBase, i(gksubfundG), \
	giDBControlBase+1, i(gksub3rdG), \
	giDBControlBase+2, i(gkfundG), \
	giDBControlBase+3, i(gk2ndG), \
	giDBControlBase+4, i(gk3rdG), \
	giDBControlBase+5, i(gk4thG), \
	giDBControlBase+6, i(gk5thG), \
	giDBControlBase+7, i(gk6thG), \
	giDBControlBase+8, i(gk8thG), \
	giVibrSwCC,        givibrG*127


endin


/****************************************************************************
 This instrument acts as the "foot switch" controlling rotor speeds.
 Can be invoked from a Score Event to set speeds directly, or via
 MIDI ch.3 from lowest 8 of 88-keys (A..E) to set via table.
 A p4 value of 0..7 and omitted p5 in a score event will use the table also. 
****************************************************************************/

instr Rotor

pset 0, 0, 2, 0, -1	; defaults for midi or quick setting
ka active "Rotor"
if ka > 1 then	; allow quick switching (abort previous)
	turnoff2 "Rotor", 1, 0
endif
p3 = giRampTimeLow > giRampTimeHigh ? giRampTimeLow : giRampTimeHigh
xtratim p3 
ishorn = gispeedhorn
isbass = gispeedbass
ichn midichn

if (p5 >= 0) then	; full-function speed from score
	gispeedhorn init   p4
	gispeedbass init   p5
else
	if (ichn == 0) then	; not from midi
		indx init   p4          ; new speed index from score
	else
		ikey notnum
		indx init (ikey - giLeslieBase)     ; new speed from K/B
                             		        ;(default: lowest of 88 keys -- MIDI 21-28)
	endif
	gispeedhorn table indx, giLeslieHorn
	gispeedbass table indx, giLeslieBass
endif

gkLrevs    linseg ishorn,giRampTimeHigh,gispeedhorn,.01,gispeedhorn ;High freq. rotor acceleration
gkLrevslow linseg isbass,giRampTimeLow,gispeedbass,.01,gispeedbass ;Low freq. rotor acceleration

endin



/****************************************************************************
 Organ ("Upper"/"Swell") Manual (MIDI input ch. 1)
****************************************************************************/

;------- A local sub-instrument is used for percussion control -----

instr percussion_envelope
	;prints "percussion entered\\n"
	p3 = 0.001
	ihold
	kount active "ManualS"
	if (kount == 0) then
		;printks "percussion switching off\\n", 1
		gkpercenv = 0
		turnoff
	endif
ipercamp init 0
if (giPercuss2 != 0) then
	ipercamp init giPercuss2*giPercAmp
elseif (giPercuss3 != 0) then
	ipercamp init giPercuss3*giPercAmp
endif
if giPercDecay > 0 then
	iperctime = giPercFast
else
	iperctime = giPercSlow
endif
;prints "percussion amp %f decay %f\n", ipercamp, iperctime
gkpercenv init ipercamp
; Decay is exponential:
if ipercamp > 0.0 then
 gkpercenv expon  ipercamp, iperctime, 0.001
endif
endin

;-------- The organ voices --------

instr ManualS

; Drawbar input from MIDI
midicontrolchange giDBControlBase, gksubfund
midicontrolchange giDBControlBase+1, gksub3rd
midicontrolchange giDBControlBase+2, gkfund
midicontrolchange giDBControlBase+3, gk2nd
midicontrolchange giDBControlBase+4, gk3rd
midicontrolchange giDBControlBase+5, gk4th
midicontrolchange giDBControlBase+6, gk5th
midicontrolchange giDBControlBase+7, gk6th
midicontrolchange giDBControlBase+8, gk8th
midicontrolchange giVibrSwCC,        givibrS, 0, 1

; Start percussion if needed:
instcount active "ManualS"	; only first key-down should trigger envelope
konce init instcount	; we need to change at k-time
;printks " konce %d giPercuss2 %d giPercuss3 %d\n", 1,  konce,  giPercuss2, giPercuss3 
if (konce == 1 && (giPercuss2 + giPercuss3) != 0) then
	event "i", "percussion_envelope", 0, 0
	konce = 0
endif

  midinoteonkey p4, p5
  inn = p4
; *** This version has fixed key velocity, as in a real organ.
;     If you want velocity input, you can modify the old code:
;  iamp = p5*0dbfs/127	-- not used
;  kswell midic7 11, 0, iamp, giSwell	-- use global CC11 ch. 1
;  printks "gkSwellPedal %f\n", 1, gkSwellPedal

ikey     init  inn-36	; ikey value '0' is lowest on organ manual
if ikey < gikeylo || ikey > gikeyhi then
	iamp=0
	turnoff
	endif

iks3 init ikey+19	;  Sub3rd
ikf init ikey+12	; Fund
ik2 init ikey+24	; Octv
ik3 init (ikey > 59? ikey+19 : ikey+31)	; 3rd
ik4 init (ikey > 54? ikey+24 : ikey+36)	; 4th
ik5 init (ikey > 50? ikey+28 : ikey+40)	; 5th
ik6 init (ikey>47? ik3 : ikey+43)	; 6th
ik8 init (ikey>42? ik4 : ikey+48)	; 8th

ifsf table ikey, giTones	; SubFund
ifs3 table iks3, giTones	;  Sub3rd
iff table  ikf, giTones	; Fund
if2 table  ik2, giTones	; Octv
if3 table  ik3, giTones	; 3rd
if4 table  ik4, giTones	; 4th
if5 table  ik5, giTones	; 5th
if6 table  ik6, giTones	; 6th
if8 table  ik8, giTones	; 8th
; prints "sub %f sub3 %f fund %f 2nd %f\\n 3rd %f 4th %f 5th %f\\n 6th %f 8th %f\\n",\
;   ifsf, ifs3, iff, if2, if3, if4, if5, if6, if8
; print ifsf, ifs3, iff, if2, if3, if4, if5, if6, if8

issf table ikey, giScaling	; SubFund
iss3 table iks3, giScaling	;  Sub3rd
isf table  ikf, giScaling	; Fund
is2 table  ik2, giScaling	; Octv
; For the higher keys, we ignore folding
is3 table  ikey+31, giScaling	; 3rd
is4 table  ikey+36, giScaling	; 4th
is5 table  ikey+40, giScaling	; 5th
is6 table  ikey+43, giScaling	; 6th
is8 table  ikey+48, giScaling	; 8th
; print issf, iss3, isf, is2, is3, is4, is5, is6, is8

iwheel1  init  (ikey >= 12 ? giSine1:giSine2)  ;The lowest 12 tone wheels have
                                               ;increased odd harmonic content.
iwheel2  init  giSine1 	; never uses lowest 12 wheels                      
  
;------------------------------------------------------------------------

; Map drawbar position to amplitude factor
ksubfund table gksubfund, giDrawbar
ksub3rd table gksub3rd, giDrawbar
kfund table gkfund, giDrawbar
k2nd table gk2nd, giDrawbar
k3rd table gk3rd, giDrawbar
k4th table gk4th, giDrawbar
k5th table gk5th, giDrawbar
k6th table gk6th, giDrawbar
k8th table gk8th, giDrawbar

;printf "drawbar pos: %d  level = %f\n", gkfund+11, gkfund, kfund

;------------------------------------------------------------------------
; Phase Management
; we use performance time and frequency to set the initial phase of
; each harmonic.
; This should both keep multiple uses of the same 'wheel' in sync,
; and provide a more realistic 'key-click' (if enabled)

iT times

giActiveS active "ManualS"
giActiveG active "ManualG"
if (giActiveS + giActiveG) == 1 then
	giBaseTime = int(iT)
endif

iT = iT - giBaseTime	; keep ref time short fort computation accuracy

;------------------------------------------------------------------------

;amp linenr gkSwellPedal, giClickRise, .01, .01	; has audible 'off' click on low harmonics
amp linenr gkSwellPedal, giClickRise, .03, .001

;------------------------------------------------------------------------
asubfund oscil  ksubfund*issf, ifsf,  iwheel1, frac(iT*ifsf)   ;The organ tone is
asub3rd  oscil  ksub3rd*iss3,  ifs3,  iwheel2, frac(iT*ifs3)   ;made from adding
afund    oscil  kfund*isf,    iff,   iwheel2, frac(iT*iff)     ;the weighted output
a2nd     oscil  k2nd*is2,     if2,   iwheel2, frac(iT*if2)     ;of 9 tone wheels
a3rd     oscil  k3rd*is3,     if3,   iwheel2, frac(iT*if3)  
a4th     oscil  k4th*is4,     if4,   iwheel2, frac(iT*if4)
a5th     oscil  k5th*is5,     if5,   iwheel2, frac(iT*if5)
a6th     oscil  k6th*is6,     if6,   iwheel2, frac(iT*if6)
a8th     oscil  k8th*is8,     if8,   iwheel2, frac(iT*if8)

; Key Click 'hash noise'
kclickenv linseg 0, .005, giClickNoise, .01, 0, .01, 0
aclick   noise   kclickenv, giClickBeta	;adjust last param for low-pass

; Percussion:
; Actual Hammond percussion is "Single triggered".
; The harmonic used for the 'percussion' sound is also available to drawbars.
; The percussion signal bypasses vibrato (as in the real circuitry)
;(In the hardware, the 9th harmonic busbar is diverted to trigger percussion, so
; that frequency is not available.  The simulation doesn't bother; for realism,
; just make sure that drawbar is set to zero.)

; ... only one of giPercuss2 or giPercuss3 should be non-zero:
; (In this version those are either 0 or 1; amplitude is from giPercAmp)
ifperc init if2
isperc init 0
if (giPercuss2 != 0) then
	isperc init  is2*giKeyVol
elseif (giPercuss3 != 0) then
	isperc init  is3*giKeyVol
	ifperc init if3
endif

ixperc = frac(iT*ifperc)
aperc     oscil  gkpercenv*isperc,  ifperc,  iwheel2, ixperc

aorgan sum  asubfund,asub3rd,afund,a2nd,a3rd,a4th,a5th,a6th,a8th,aclick
aorgan   =      amp*aorgan*giKeyVol
gaorgan  =      gaorgan+givibrS*aorgan
gabypass =      gabypass+(gkOne-givibrS)*aorgan+aperc*amp + aorgan*aclick

;;; debug:
; kpip metro 1
; kmaxorg max_k gaorgan, kpip, 1
; kmaxbyp max_k gabypass, kpip, 1
; if (kmaxorg + kmaxbyp) > 1 then
;  printks "at tone gen %f + %f\n", 1, kmaxorg, kmaxbyp
; endif

	endin


/****************************************************************************
 Organ Second ("Lower"/"Great") Manual (MIDI input ch. 2)
****************************************************************************/

instr ManualG

  midinoteonkey p4, p5
  inn = p4
;  iamp = p5*0dbfs/127	-- not used
;  kswell midic7 11, 0, iamp, giSwell -- use global...
;  printks "gkSwellPedal %f\n", 1, gkSwellPedal

ikey     init  inn-36
; print inn, ipch, ikey
if ikey < gikeylo || ikey > gikeyhi then
	iamp=0
	turnoff
	endif
iks3 init ikey+19	;  Sub3rd
ikf init ikey+12	; Fund
ik2 init ikey+24	; Octv
ik3 init (ikey > 59? ikey+19 : ikey+31)	; 3rd
ik4 init (ikey > 54? ikey+24 : ikey+36)	; 4th
ik5 init (ikey > 50? ikey+28 : ikey+40)	; 5th
ik6 init (ikey>47? ik3 : ikey+43)	; 6th
ik8 init (ikey>42? ik4 : ikey+48)	; 8th

ifsf table ikey, giTones	; SubFund
ifs3 table iks3, giTones	;  Sub3rd
iff table  ikf, giTones	; Fund
if2 table  ik2, giTones	; Octv
if3 table  ik3, giTones	; 3rd
if4 table  ik4, giTones	; 4th
if5 table  ik5, giTones	; 5th
if6 table  ik6, giTones	; 6th
if8 table  ik8, giTones	; 8th
; print ifsf, ifs3, iff, if2, if3, if4, if5, if6, if8

issf table ikey, giScaling	; SubFund
iss3 table iks3, giScaling	;  Sub3rd
isf table  ikf, giScaling	; Fund
is2 table  ik2, giScaling	; Octv
; For the higher keys, we ignore folding
is3 table  ikey+31, giScaling	; 3rd
is4 table  ikey+36, giScaling	; 4th
is5 table  ikey+40, giScaling	; 5th
is6 table  ikey+43, giScaling	; 6th
is8 table  ikey+48, giScaling	; 8th
; print issf, iss3, isf, is2, is3, is4, is5, is6, is8

iwheel1  init  (ikey >= 12 ? giSine1:giSine2)  ;lowest 12 tone wheels
iwheel2  init  giSine1                       

;------------------------------------------------------------------------
midicontrolchange giDBControlBase, gksubfundG
midicontrolchange giDBControlBase+1, gksub3rdG
midicontrolchange giDBControlBase+2, gkfundG
midicontrolchange giDBControlBase+3, gk2ndG
midicontrolchange giDBControlBase+4, gk3rdG
midicontrolchange giDBControlBase+5, gk4thG
midicontrolchange giDBControlBase+6, gk5thG
midicontrolchange giDBControlBase+7, gk6thG
midicontrolchange giDBControlBase+8, gk8thG
midicontrolchange giVibrSwCC,        givibrG, 0, 1

ksubfund table gksubfundG, giDrawbar
ksub3rd table gksub3rdG, giDrawbar
kfund table gkfundG, giDrawbar
k2nd table gk2ndG, giDrawbar
k3rd table gk3rdG, giDrawbar
k4th table gk4thG, giDrawbar
k5th table gk5thG, giDrawbar
k6th table gk6thG, giDrawbar
k8th table gk8thG, giDrawbar

;------------------------------------------------------------------------

iT times

giActiveS active "ManualS"
giActiveG active "ManualG"
if (giActiveS + giActiveG) == 1 then
	giBaseTime = int(iT)
endif

iT = iT - giBaseTime	; keep ref time short fort computation accuracy

;------------------------------------------------------------------------
;amp linenr gkSwellPedal, giClickRise, .01, .01	; has audible 'off' click on low harmonics
amp linenr gkSwellPedal, giClickRise, .03, .001

; Percussion not available on lower manual
;------------------------------------------------------------------------
asubfund oscil  ksubfund*issf, ifsf,  iwheel1, frac(iT*ifsf)
asub3rd  oscil  ksub3rd*iss3,  ifs3,  iwheel2, frac(iT*ifs3)
afund    oscil  kfund*isf,    iff,   iwheel2, frac(iT*iff) 
a2nd     oscil  k2nd*is2,     if2,   iwheel2, frac(iT*if2) 
a3rd     oscil  k3rd*is3,     if3,   iwheel2, frac(iT*if3) 
a4th     oscil  k4th*is4,     if4,   iwheel2, frac(iT*if4)
a5th     oscil  k5th*is5,     if5,   iwheel2, frac(iT*if5)
a6th     oscil  k6th*is6,     if6,   iwheel2, frac(iT*if6)
a8th     oscil  k8th*is8,     if8,   iwheel2, frac(iT*if8)

; Key Click 'hash noise'
kclickenv linseg 0, .005, giClickNoise, .01, 0, .01, 0
aclick   noise   kclickenv, giClickBeta	;adjust last param for low-pass

aorgan sum  asubfund,asub3rd,afund,a2nd,a3rd,a4th,a5th,a6th,a8th
aorgan   =      amp*aorgan*giKeyVol
gaorgan  =      gaorgan+givibrG*aorgan
gabypass =      gabypass+(gkOne-givibrG)*aorgan + aorgan*aclick

endin


/****************************************************************************
 This instrument adds Vibrato (now always running)
	p4 = effect  (O=off, 1=Vibrato, 0.5=Chorus)
	p5 = depth -- i.e fraction of delay-line scanned
		 (Hammond values V/C1=0.45, V/C2=0.66, V/C3=1.0)
 These parameters are fairly experimental -- adjust to taste:
	p6 = delay-zero max cutoff freq
	p7 = delay freq adjustment factor (multiplier to DL scan)
	p8 = lowpass filter Q
	p9 = "Buzz" amplitude
****************************************************************************/

instr Vibrato

; These initializations don't work under C5 once vibrato is running
; Use Instr "Control" to alter them -- or MIDI via Output instrument
	givibchor	init (p4 > 1? 1 : p4)	; 0..1
	giscanfrac	init (p5 > 1? 1 : p5)	; 0..1 
	giftopfrq	init p6					; ~10000
	gifscale	init p7					; ~2??
	giQ			init p8					;~2
	givrchor 	init 1 - givibchor
	giBuzzAmp	init p9					; maybe 0.1

	; scanner shaft speed =  412 RPM => 6.87 Hz
	kscan oscili giscanfrac, 6.87, giScanner	;triangle wave
	ascan interp kscan*giVibrLine
	abuzz oscils giBuzzAmp, 109.92, 0	; stator vane wobble
	abuzz = abuzz + 1.0
	avibrout vdelay gaorgan, ascan, giVibrLine*2
	; delay line attenuates higher frequencies
	afiltout lowpass2 avibrout*abuzz, giftopfrq/(1+kscan*gifscale), giQ
	gaorgan mac givrchor,gaorgan, givibchor,afiltout, gkOne, gabypass
	gabypass = 0	;zeroed after first use

endin


/****************************************************************************
 "Spring Reverb" (before amplifier & Leslie)
 -- always running; can only be set via Control instrument (no MIDI)
****************************************************************************/


instr   Spring_Reverb

;	impulse ftgen 0, 0, 0, 1, "spring_reverb.wav", 0, 0, 0
	impulse ftgen 0, 0, 0, 1, "spring_reverb24.raw", 0, 6, 0	; 24000, float
	impulselen = nsamp(impulse)
;	ipartitionsize = 256
	ipartitionsize = 2048

	klastrvb init i(gkSpringRvrb)
	krevb ctrl7, giChanLeslie, giReverbBase, 0, 1
	if krevb != klastrvb then
		gkSpringRvrb = krevb
		klastrvb = krevb
	endif
	
	kLevel = 0.1*gkSpringRvrb
                     			
	gaorgan = gaorgan+gabypass	; in case vibrato is off
	gabypass = 0

	awet ftconv kLevel*gaorgan, impulse, ipartitionsize, 0, impulselen
	gaorgan = (1-gkSpringRvrb)*gaorgan + awet

endin
    

/****************************************************************************
 Distortion Stage
****************************************************************************/

; Amplifier Overdrive Simulation:

instr Overdrive
	
	pset 0, 0, 0, 0
	initc7 giChanLeslie, giOverdriveCC, p4/4	; initc7 range is 0..1
	gkOverDrive init p4	; 1..4 = degree, 0 = OFF
	gkOverDrive ctrl7, giChanLeslie, giOverdriveCC, 0, 4	; rescaled to max = 4
	; printk2 gkOverDrive

	; Overdrive distortion -- after Peter Billam's pjbsynth
	; -- in turn based on Steve Harris' LADSPA plugin 'foverdrive 1196':
	; drivem1 = drive - 1.0 ; fx = fabs(x);
	; output   x*(fx + drive)/(x*x + drivem1*fx + 1.0);
	gaorgan = gaorgan+gabypass
	gabypass = 0	;zeroed after first use
	if gkOverDrive > 0.95 then
		kDm1 = gkOverDrive - 1
		aAbs = abs(gaorgan)
		gaorgan = gaorgan * ( aAbs + gkOverDrive ) \
			/ ( gaorgan*gaorgan + kDm1*aAbs + 1.0 )
	endif
	
endin



/****************************************************************************
 Rotating Leslie Speaker
 Multi-cabinet-reflection version
****************************************************************************/

instr Leslie

	p3 = p3+0.1	; so ihold will never see a zero p3
	ihold	; in case we restarted...	
	gkLeslieOn init 1

  if gkLeslieOn == 0 then
    ; Turn the Leslie off now.
    printks "stopping Leslie\n", 0
    gkLeslieOn = 0
    turnoff
  endif

if p4<=1 then	; the limitations Csound syntax... sigh...
Sdefl = "Leslie starting with Deflectors -- Quad mix = %.2f, Refl delay = %.2f msec\n"
else
Sdefl = "Leslie starting with No Deflectors -- Quad mix = %.2f, Refl delay = %.2f msecc\n"
endif
prints Sdefl, p5, p6

; p4 is <=1 to use deflectors, >1 for no deflectors:
giLtaba    init   (p4 <= 1 ? giLhidef : giLhinodef)
giLtabb    init   (p4 <= 1 ? giLmiddef : giLmidnodef)
giLrefl     init   p5             ;Quadrature (sideways) mix amount
giLquadlag     init   p6/1000             ;p6 is reflection delay in *millisecs*
giLboxwidth init  0.0007          ;secondary reflection delay
giLboxdepth init  0.0005          ;cabinet-back reflection delay
giLradius  init   0.00025         ;Radius of the rotating horn.
;giLradlow  init   0.00035         ;Radius of the rotating scoop.
;giLradius  init   0.0004         ;Radius of the rotating horn.
giLradlow  init   0.0007         ;Radius of the rotating scoop.
;Based on a 23 cm radius horn (~9") with speed of sound = 33000 cm/s:
; ... is way too large! (Why?)
;giLradius  init   .0007         ;Radius of the rotating horn.
;giLradlow  init   .0010         ;Radius of the rotating scoop -- a bit bigger.

;idelayline init 2*(giLradlow + giMinDelay) + giLquadlag

idelayline  init   2.0            ;Length of delay line (longish to allow for reflections).
idelpivot = idelayline - (giLradlow >= giLradius ? giLradlow : giLradius) - 2*giMinDelay

anull init 0	; for feeding delay line

;------------------------------------------------------------------------
; Higher frequencies are assumed to reflect from walls so Doppler
; will be shifted by +-90 degrees

arotat    oscil   giLradius, gkLrevs, giSine1 
arotatq   oscil   giLradius, gkLrevs, giSine1, 0.25 
adopdirect   =    idelpivot + arotat
adopback   =    idelpivot - arotat -giLboxdepth
adopquadl   =     (idelpivot + arotatq) - giLquadlag
adopquadr   =     (idelpivot - arotatq) - giLquadlag
;------------------------------------------------------------------------
; Lower speaker is mostly Amplitude Modulation, so Doppler isn't split

arotatlow  oscil   giLradlow, gkLrevslow, giSine1
adopllow  =        idelpivot + arotatlow

;------------------------------------------------------------------------
ireflhi = giHiRefl*giLrefl	; overall level of reflected high freqs	
ireflmid = giMidRefl*giLrefl	; overall level of reflected mid freqs
ilowall = giLowAll * (1 + giLrefl)	; low freqs don't go through reflection	

kampdrcthi    oscil    giHiDirect, gkLrevs, giLtaba
kampbackhi    oscil    ireflhi, gkLrevs, giLtaba, 0.50
kamplefthi    oscil    ireflhi, gkLrevs, giLtaba, 0.25
kamprighthi   oscil    ireflhi, gkLrevs, giLtaba, 0.75
kampdrctmid   oscil    giMidDirect,gkLrevs,giLtabb
kampbackmid   oscil    ireflmid,gkLrevs,giLtabb,0.50
kampleftmid   oscil    ireflmid,gkLrevs,giLtabb,0.25
kamprightmid  oscil    ireflmid,gkLrevs,giLtabb,0.75
kamplow       oscil    ilowall,gkLrevslow,giLlowrot
  
afhi     butterbp gaorgan,5000,4000     ;Divide the frequency into three
;afmid    butterbp gaorgan,2000,3000    
afmid    butterbp gaorgan,2000,1500    
aflow    butterlp gaorgan,500        ;of different frequencies.

afleftq =  afhi*kamplefthi + afmid*kampleftmid
afrightq = afhi*kamprighthi + afmid*kamprightmid
;------------------------------------------------------------------------
; Left Channel delay line

galeft delayr  idelayline,1	; fed with various inputs

	deltapxw  afhi*kampdrcthi + afmid*kampdrctmid, adopdirect, 4
	deltapxw  afhi*kampbackhi + afmid*kampbackmid, adopback, 4
	deltapxw  afleftq, adopquadl, 4
	deltapxw  afrightq*giLrefl, adopquadr - giLboxwidth, 4
	deltapxw  aflow*kamplow, adopllow, 4

        delayw  anull	; nothing actually fed to head of delayline

;------------------------------------------------------------------------
; Right Channel delay line

garight delayr  idelayline,1	; fed with various inputs

	deltapxw  afhi*kampdrcthi + afmid*kampdrctmid, adopdirect, 4
	deltapxw  afhi*kampbackhi + afmid*kampbackmid, adopback, 4
	deltapxw  afrightq, adopquadr, 4
	deltapxw  afleftq*giLrefl, adopquadl - giLboxwidth, 4
	deltapxw  aflow*kamplow, adopllow, 4

        delayw  anull	; nothing actually fed to head of delayline

;------------------------------------------------------------------------

endin



/****************************************************************************
 The end of the audio chain
 -- must be running continuously for audio output
 'Reverb' (p4) can be adjusted by a 'Control' command at any time
  or via CC26 ch. 3
****************************************************************************/

instr Output

/***********************************

;; Debug if needed:

	kstatus, kchan, kdata1, kdata2 midiin
	printf "status %d chan %d data1 %d data2 %d\n", kstatus, kstatus, kchan, kdata1, kdata2
	kount active "ManualS"
	printks "ManualS => %d\n", 1, kount
	kount active "ManualG"
	printks "ManualG => %d\n", 1, kount
	kount active "Vibrato"
	printks "Vibrato => %d\n", 1, kount
	kount active "Spring_Reverb"
	printks "Spring_Reverb => %d\n", 1, kount
	kount active "Overdrive"
	printks "Overdrive => %d\n", 1, kount
	kount active "Leslie"
	printks "Leslie => %d\n", 1, kount
	
************************************/

;-----------------------------------------------------------------

; MIDI Swell Pedal handling (CC11)

gkSwellPedal ctrl7 giChanSwell, 11, 0, 1, giSwell
	
;-----------------------------------------------------------------

; MIDI Controller handling -- Percussion Switching (Ch.1 CC80/81/82)

klastOn init 0
klastHarm init 0
klastDecay init 0

kpercOn ctrl7 giChanSwell, giPercSwBase, 0, 1
kpercHarm ctrl7 giChanSwell, giPercSwBase+1, 0, 1
kpercDecay ctrl7 giChanSwell, giPercSwBase+2, 0, 1

if kpercOn != klastOn || kpercHarm != klastHarm || kpercDecay != klastDecay  then
  event "i", "Percussion", 0, 0, kpercOn, kpercHarm, kpercDecay
  klastOn = kpercOn
  klastHarm = kpercHarm
  klastDecay = kpercDecay
endif


; MIDI Controller handling -- Percussion Scale/Fast/Slow (Ch.3 CC22/23/24)

iminScale init 0.5
imaxScale init 10
iminFast init 0.01
imaxFast init 0.5
iminSlow init 1.0
imaxSlow init 10.0

klastscale init iminScale
klastfast init iminFast
klastslow init iminSlow

kPercScale ctrl7 giChanLeslie, giPercParam, iminScale, imaxScale
kPercFast ctrl7 giChanLeslie, giPercParam+1, iminFast, imaxFast
kPercSlow ctrl7 giChanLeslie, giPercParam+2, iminSlow, imaxSlow

if kPercScale != klastscale then
	event "i", "SetPercParams", 0, 0, kPercScale, giPercFast, giPercSlow
	klastscale = kPercScale
endif

if kPercFast != klastfast then
	event "i", "SetPercParams", 0, 0, giPercAmp, kPercFast, giPercSlow
	klastfast = kPercFast
endif

if kPercSlow != klastslow then
	event "i", "SetPercParams", 0, 0, giPercAmp, giPercFast, kPercSlow
	klastslow = kPercSlow
endif


;-----------------------------------------------------------------

; MIDI Controller handling -- Vibrato/Chorus (Ch.3 CC20)

; '64'=>OFF, <64 => Chorus, >64 => Vibrato
; Level: 1 = 43/86, 2 = 22/106, 3 = 0/127

	klastvib init giInitVib
	ifrac ftgenonce 0, 0, 4, 2, 0.0, 0.4, 0.7, 1.0
	kvib ctrl7, giChanLeslie, giVibratoCC, 0, 127
	if kvib != klastvib then
		klastvib = kvib
		kindex = round(abs((kvib-64)/64)*3)
		kfrac table kindex, ifrac
		if kvib == 64 then
			printks "setting Vibrato/Chorus Off\n", 1
			kfx = 0
		elseif kvib > 64 then
			printks "setting Vibrato to %d\n", 1, kindex
			kfx = 1
		else
			printks "setting Chorus to %d\n", 1, kindex
			kfx = 0.5
		endif
		event "i", "Control", 0, 5, kfx, kfrac
	endif

;-----------------------------------------------------------------	


	if (gkLeslieOn == 0) then
		galeft = (gaorgan+gabypass)
		garight = galeft
	endif


	/** Stereo reverb: **/
	klastrvb init i(gkRoomRvrb)
	krevb ctrl7, giChanLeslie, giReverbBase+1, 0, 0.95, giRvbRamp	; best >= 0.6
;	printks "krevb=%f\n", 1, krevb
	if krevb != klastrvb then
		gkRoomRvrb = krevb
		klastrvb = krevb
	endif
	
	if (gkRoomRvrb > 0) then
		arvbl, arvbr reverbsc galeft, garight, gkRoomRvrb, 12000
	else
		arvbl = 0
		arvbr = 0
	endif
	outs giOutGain*(galeft+arvbl), giOutGain*(garight+arvbr)

	;-----------------------------------------
	/********************	
	;Amplitude monitor -- disabled... causes audio glitches
	
	all = giOutGain*(galeft+arvbl + garight+arvbr)
	ktrig	metro	1
	kmax	max_k	all, ktrig, 1
	krms rms all
	kratio = krms > 0.01 ? kmax/krms : 1
	printf "Abs amp peak=%g rms=%g  (ratio=%g)\n", kmax, kmax, krms, kratio

	********************/	
	;-----------------------------------------
	
    gaorgan = 0
	gabypass = 0
    galeft = 0
    garight = 0


endin



/****************************************************************************
 "Control" Instrument used to reset parameters of running instruments via Score events
	p3 = instrument to be affected (NOT duration! -- instrument is never sustained):
	  These may change...:
		5 = Vibrato params (all must be present to avoid zeros)
		6 = Spring reverb level (0..1)
		7 = Overdrive level (0..4)
		8 = Leslie Parameters (must turn off (p4=0) before resetting)
		10 = Room Reverb
		11 = Key Click
		12 = Leslie/Room freq response
	p4... = parameters appropriate to instrument
****************************************************************************/

instr Control
	
	pset 0, 0, 0, 0, 0, 0, 0	;
	if p3 == 5 then	; Vibrato
		givibchor	init (p4 > 1? 1 : p4)	; 0..1
		givrchor 	init 1 - givibchor
		giscanfrac	init (p5 > 1? 1 : p5)	; 0..1
		if p6 > 0 then
			giftopfrq	init p6					; ~10000
		endif
		if p7 > 0 then
			gifscale	init p7					; ~2??
		endif
		if p8 > 0 then
			giQ			init p8					;~2
		endif
		if p9 > 0 then
			giBuzzAmp	init p9					; maybe 0.1
		endif
	elseif p3 == 6 then	; Spring Reverb
		gkSpringRvrb	init (p4 > 1? 1 : p4)	; 0..1 (for now)
	elseif p3 == 7 then	; Overdrive level
		; need to set the MIDI controller (otherwise just gets reset)
		initc7 giChanLeslie, giOverdriveCC, p4 > 4 ? 1 : p4/4	; range is 0..1
	elseif p3 == 8 then	; Leslie
		gkLeslieOn init 0	; turn off first in all cases
		if p4 > 0 then	;restart Leslie (after waiting a mo')
			event "i", "Leslie", 0.05, 0, p4, p5, p6
		endif
;	elseif p3 == 9 then -- not used currently
	elseif p3 == 10 then	; Reverb -- now range 0..<1.0
		gkRoomRvrb init p4 > 0.95 ? 0.95 : p4	; best >= 0.6
	elseif p3 == 11 then	; Key Click
		giClickRise init p4
		giClickNoise init p5
		if p6 > 0 then
			giClickBeta init p6
		endif
	elseif p3 == 12 then	; Leslie freq resp.
		giHiDirect init p4
		giHiRefl init p5
		giMidDirect init p6
		giMidRefl init p7
		giLowAll init p8
	else
		prints "Use p3 =:\n"
		prints "   5 for Vibrato\n"
		prints "   6 for Spring Reverb\n"
		prints "   7 for Overdrive\n"
		prints "   8 for Leslie\n"
		prints "  10 for Room Reverb\n"
		prints "  11 for Key Click\n"
		prints "  12 for Frequncy Response\n"
	endif
	
	turnoff	; always
endin

</CsInstruments>

<CsScore>
;-------------------------------------------------------------------
; Tone Wheel Organ with Rotating Speaker v8.1
; Original by Hans Mikelson
; mods by Pete Goodeve

f0  100000         ; Continue running a long time for midi data input.


; drawbars/vibrato on/percussion 0ff (2nd, Slow)
i"DrawbarsS" 0 0     8   8   5   3   2   4   5   8   8     1      0    0    0

; drawbars/vibrato on-off Lower Manual (no perc needed)
i "DrawbarsG" 0 0     8   8   8   0   0   0   0   0   0     1

; Leslie speed (Horn and Bass Rotor speeds are different)
;  -- note 2 sec 'ramp' to speed
; changes can also be made via MIDI ch.3 (see orc for details)
; It is reported that actual Fast Leslie is close to:
;                Ramp   Horn   Bass
i "Rotor"    0    2     6.8    6.6

</CsScore>

</CsoundSynthesizer>
 
